@@ -23,10 +23,11 @@ from ..messages import (
     ModelResponse,
 )
 from ..settings import ModelSettings
-from ..usage import Usage
 from . import KnownModelName, Model, ModelRequestParameters, StreamedResponse
 from .wrapper import WrapperModel
 
+__all__ = 'instrument_model', 'InstrumentationSettings', 'InstrumentedModel'
+
 MODEL_SETTING_ATTRIBUTES: tuple[
     Literal[
         'max_tokens',
@@ -49,6 +50,17 @@ MODEL_SETTING_ATTRIBUTES: tuple[
 ANY_ADAPTER = TypeAdapter[Any](Any)
 
 
+def instrument_model(model: Model, instrument: InstrumentationSettings | bool) -> Model:
+    """Instrument a model with OpenTelemetry/logfire."""
+    if instrument and not isinstance(model, InstrumentedModel):
+        if instrument is True:
+            instrument = InstrumentationSettings()
+
+        model = InstrumentedModel(model, instrument)
+
+    return model
+
+
 @dataclass(init=False)
 class InstrumentationSettings:
     """Options for instrumenting models and agents with OpenTelemetry.
@@ -65,6 +77,7 @@ class InstrumentationSettings:
     tracer: Tracer = field(repr=False)
     event_logger: EventLogger = field(repr=False)
     event_mode: Literal['attributes', 'logs'] = 'attributes'
+    include_binary_content: bool = True
 
     def __init__(
         self,
@@ -72,6 +85,7 @@ class InstrumentationSettings:
         event_mode: Literal['attributes', 'logs'] = 'attributes',
         tracer_provider: TracerProvider | None = None,
         event_logger_provider: EventLoggerProvider | None = None,
+        include_binary_content: bool = True,
     ):
         """Create instrumentation options.
 
@@ -85,6 +99,7 @@ class InstrumentationSettings:
                 If not provided, the global event logger provider is used.
                 Calling `logfire.configure()` sets the global event logger provider, so most users don't need this.
                 This is only used if `event_mode='logs'`.
+            include_binary_content: Whether to include binary content in the instrumentation events.
         """
         from pydantic_ai import __version__
 
@@ -93,6 +108,40 @@ class InstrumentationSettings:
         self.tracer = tracer_provider.get_tracer('pydantic-ai', __version__)
         self.event_logger = event_logger_provider.get_event_logger('pydantic-ai', __version__)
         self.event_mode = event_mode
+        self.include_binary_content = include_binary_content
+
+    def messages_to_otel_events(self, messages: list[ModelMessage]) -> list[Event]:
+        """Convert a list of model messages to OpenTelemetry events.
+
+        Args:
+            messages: The messages to convert.
+
+        Returns:
+            A list of OpenTelemetry events.
+        """
+        events: list[Event] = []
+        instructions = InstrumentedModel._get_instructions(messages)  # pyright: ignore [reportPrivateUsage]
+        if instructions is not None:
+            events.append(Event('gen_ai.system.message', body={'content': instructions, 'role': 'system'}))
+
+        for message_index, message in enumerate(messages):
+            message_events: list[Event] = []
+            if isinstance(message, ModelRequest):
+                for part in message.parts:
+                    if hasattr(part, 'otel_event'):
+                        message_events.append(part.otel_event(self))
+            elif isinstance(message, ModelResponse):  # pragma: no branch
+                message_events = message.otel_events()
+            for event in message_events:
+                event.attributes = {
+                    'gen_ai.message.index': message_index,
+                    **(event.attributes or {}),
+                }
+            events.extend(message_events)
+
+        for event in events:
+            event.body = InstrumentedModel.serialize_any(event.body)
+        return events
 
 
 GEN_AI_SYSTEM_ATTRIBUTE = 'gen_ai.system'
@@ -122,11 +171,11 @@ class InstrumentedModel(WrapperModel):
         messages: list[ModelMessage],
         model_settings: ModelSettings | None,
         model_request_parameters: ModelRequestParameters,
-    ) -> tuple[ModelResponse, Usage]:
+    ) -> ModelResponse:
         with self._instrument(messages, model_settings, model_request_parameters) as finish:
-            response, usage = await super().request(messages, model_settings, model_request_parameters)
-            finish(response, usage)
-            return response, usage
+            response = await super().request(messages, model_settings, model_request_parameters)
+            finish(response)
+            return response
 
     @asynccontextmanager
     async def request_stream(
@@ -143,8 +192,8 @@ class InstrumentedModel(WrapperModel):
                 ) as response_stream:
                     yield response_stream
             finally:
-                if response_stream:
-                    finish(response_stream.get(), response_stream.usage())
+                if response_stream:  # pragma: no branch
+                    finish(response_stream.get())
 
     @contextmanager
     def _instrument(
@@ -152,7 +201,7 @@ class InstrumentedModel(WrapperModel):
         messages: list[ModelMessage],
         model_settings: ModelSettings | None,
         model_request_parameters: ModelRequestParameters,
-    ) -> Iterator[Callable[[ModelResponse, Usage], None]]:
+    ) -> Iterator[Callable[[ModelResponse], None]]:
         operation = 'chat'
         span_name = f'{operation} {self.model_name}'
         # TODO Missing attributes:
@@ -177,12 +226,12 @@ class InstrumentedModel(WrapperModel):
 
         with self.settings.tracer.start_as_current_span(span_name, attributes=attributes) as span:
 
-            def finish(response: ModelResponse, usage: Usage):
+            def finish(response: ModelResponse):
                 if not span.is_recording():
                     return
 
-                events = self.messages_to_otel_events(messages)
-                for event in self.messages_to_otel_events([response]):
+                events = self.settings.messages_to_otel_events(messages)
+                for event in self.settings.messages_to_otel_events([response]):
                     events.append(
                         Event(
                             'gen_ai.choice',
@@ -193,7 +242,7 @@ class InstrumentedModel(WrapperModel):
                             },
                         )
                     )
-                new_attributes: dict[str, AttributeValue] = usage.opentelemetry_attributes()  # type: ignore
+                new_attributes: dict[str, AttributeValue] = response.usage.opentelemetry_attributes()  # pyright: ignore[reportAssignmentType]
                 attributes.update(getattr(span, 'attributes', {}))
                 request_model = attributes[GEN_AI_REQUEST_MODEL_ATTRIBUTE]
                 new_attributes['gen_ai.response.model'] = response.model_name or request_model
@@ -241,9 +290,9 @@ class InstrumentedModel(WrapperModel):
             except Exception:  # pragma: no cover
                 pass
             else:
-                if parsed.hostname:
+                if parsed.hostname:  # pragma: no branch
                     attributes['server.address'] = parsed.hostname
-                if parsed.port:
+                if parsed.port:  # pragma: no branch
                     attributes['server.port'] = parsed.port
 
         return attributes
@@ -251,34 +300,13 @@ class InstrumentedModel(WrapperModel):
     @staticmethod
     def event_to_dict(event: Event) -> dict[str, Any]:
         if not event.body:
-            body = {}
+            body = {}  # pragma: no cover
         elif isinstance(event.body, Mapping):
             body = event.body  # type: ignore
         else:
             body = {'body': event.body}
         return {**body, **(event.attributes or {})}
 
-    @staticmethod
-    def messages_to_otel_events(messages: list[ModelMessage]) -> list[Event]:
-        result: list[Event] = []
-        for message_index, message in enumerate(messages):
-            message_events: list[Event] = []
-            if isinstance(message, ModelRequest):
-                for part in message.parts:
-                    if hasattr(part, 'otel_event'):
-                        message_events.append(part.otel_event())
-            elif isinstance(message, ModelResponse):
-                message_events = message.otel_events()
-            for event in message_events:
-                event.attributes = {
-                    'gen_ai.message.index': message_index,
-                    **(event.attributes or {}),
-                }
-            result.extend(message_events)
-        for event in result:
-            event.body = InstrumentedModel.serialize_any(event.body)
-        return result
-
     @staticmethod
     def serialize_any(value: Any) -> str:
         try:
