@@ -1,16 +1,16 @@
 from __future__ import annotations as _annotations
 
 import base64
-import re
 import warnings
 from collections.abc import AsyncIterable, AsyncIterator, Sequence
 from contextlib import asynccontextmanager
-from dataclasses import dataclass, field, replace
+from dataclasses import dataclass, field
 from datetime import datetime, timezone
 from typing import Any, Literal, Union, cast, overload
 
 from typing_extensions import assert_never
 
+from pydantic_ai.profiles.openai import OpenAIModelProfile
 from pydantic_ai.providers import Provider, infer_provider
 
 from .. import ModelHTTPError, UnexpectedModelBehavior, _utils, usage
@@ -33,6 +33,7 @@ from ..messages import (
     UserPromptPart,
     VideoUrl,
 )
+from ..profiles import ModelProfileSpec
 from ..settings import ModelSettings
 from ..tools import ToolDefinition
 from . import (
@@ -43,7 +44,6 @@ from . import (
     check_allow_model_requests,
     get_user_agent,
 )
-from ._json_schema import JsonSchema, WalkJsonSchema
 
 try:
     from openai import NOT_GIVEN, APIStatusError, AsyncOpenAI, AsyncStream, NotGiven
@@ -170,7 +170,9 @@ class OpenAIModel(Model):
         self,
         model_name: OpenAIModelName,
         *,
-        provider: Literal['openai', 'deepseek', 'azure', 'openrouter'] | Provider[AsyncOpenAI] = 'openai',
+        provider: Literal['openai', 'deepseek', 'azure', 'openrouter', 'grok', 'fireworks', 'together']
+        | Provider[AsyncOpenAI] = 'openai',
+        profile: ModelProfileSpec | None = None,
         system_prompt_role: OpenAISystemPromptRole | None = None,
     ):
         """Initialize an OpenAI model.
@@ -180,13 +182,17 @@ class OpenAIModel(Model):
                 [here](https://github.com/openai/openai-python/blob/v1.54.3/src/openai/types/chat_model.py#L7)
                 (Unfortunately, despite being ask to do so, OpenAI do not provide `.inv` files for their API).
             provider: The provider to use. Defaults to `'openai'`.
+            profile: The model profile to use. Defaults to a profile picked by the provider based on the model name.
             system_prompt_role: The role to use for the system prompt message. If not provided, defaults to `'system'`.
                 In the future, this may be inferred from the model name.
         """
         self._model_name = model_name
+
         if isinstance(provider, str):
             provider = infer_provider(provider)
         self.client = provider.client
+        self._profile = profile or provider.model_profile
+
         self.system_prompt_role = system_prompt_role
 
     @property
@@ -221,9 +227,6 @@ class OpenAIModel(Model):
         async with response:
             yield await self._process_streamed_response(response)
 
-    def customize_request_parameters(self, model_request_parameters: ModelRequestParameters) -> ModelRequestParameters:
-        return _customize_request_parameters(model_request_parameters)
-
     @property
     def model_name(self) -> OpenAIModelName:
         """The model name."""
@@ -401,8 +404,7 @@ class OpenAIModel(Model):
             function={'name': t.tool_name, 'arguments': t.args_as_json_str()},
         )
 
-    @staticmethod
-    def _map_tool_definition(f: ToolDefinition) -> chat.ChatCompletionToolParam:
+    def _map_tool_definition(self, f: ToolDefinition) -> chat.ChatCompletionToolParam:
         tool_param: chat.ChatCompletionToolParam = {
             'type': 'function',
             'function': {
@@ -411,7 +413,7 @@ class OpenAIModel(Model):
                 'parameters': f.parameters_json_schema,
             },
         }
-        if f.strict:
+        if f.strict and OpenAIModelProfile.from_profile(self.profile).openai_supports_strict_tool_definition:
             tool_param['function']['strict'] = f.strict
         return tool_param
 
@@ -533,18 +535,23 @@ class OpenAIResponsesModel(Model):
         self,
         model_name: OpenAIModelName,
         *,
-        provider: Literal['openai', 'deepseek', 'azure'] | Provider[AsyncOpenAI] = 'openai',
+        provider: Literal['openai', 'deepseek', 'azure', 'openrouter', 'grok', 'fireworks', 'together']
+        | Provider[AsyncOpenAI] = 'openai',
+        profile: ModelProfileSpec | None = None,
     ):
         """Initialize an OpenAI Responses model.
 
         Args:
             model_name: The name of the OpenAI model to use.
             provider: The provider to use. Defaults to `'openai'`.
+            profile: The model profile to use. Defaults to a profile picked by the provider based on the model name.
         """
         self._model_name = model_name
+
         if isinstance(provider, str):
             provider = infer_provider(provider)
         self.client = provider.client
+        self._profile = profile or provider.model_profile
 
     @property
     def model_name(self) -> OpenAIModelName:
@@ -582,9 +589,6 @@ class OpenAIResponsesModel(Model):
         async with response:
             yield await self._process_streamed_response(response)
 
-    def customize_request_parameters(self, model_request_parameters: ModelRequestParameters) -> ModelRequestParameters:
-        return _customize_request_parameters(model_request_parameters)
-
     def _process_response(self, response: responses.Response) -> ModelResponse:
         """Process a non-streamed response, and prepare a message to return."""
         timestamp = datetime.fromtimestamp(response.created_at, tz=timezone.utc)
@@ -690,15 +694,15 @@ class OpenAIResponsesModel(Model):
             tools += [self._map_tool_definition(r) for r in model_request_parameters.output_tools]
         return tools
 
-    @staticmethod
-    def _map_tool_definition(f: ToolDefinition) -> responses.FunctionToolParam:
+    def _map_tool_definition(self, f: ToolDefinition) -> responses.FunctionToolParam:
         return {
             'name': f.name,
             'parameters': f.parameters_json_schema,
             'type': 'function',
             'description': f.description,
-            # NOTE: f.strict should already be a boolean thanks to customize_request_parameters
-            'strict': f.strict or False,
+            'strict': bool(
+                f.strict and OpenAIModelProfile.from_profile(self.profile).openai_supports_strict_tool_definition
+            ),
         }
 
     async def _map_messages(
@@ -980,142 +984,3 @@ def _map_usage(response: chat.ChatCompletion | ChatCompletionChunk | responses.R
             total_tokens=response_usage.total_tokens,
             details=details,
         )
-
-
-_STRICT_INCOMPATIBLE_KEYS = [
-    'minLength',
-    'maxLength',
-    'pattern',
-    'format',
-    'minimum',
-    'maximum',
-    'multipleOf',
-    'patternProperties',
-    'unevaluatedProperties',
-    'propertyNames',
-    'minProperties',
-    'maxProperties',
-    'unevaluatedItems',
-    'contains',
-    'minContains',
-    'maxContains',
-    'minItems',
-    'maxItems',
-    'uniqueItems',
-]
-
-_sentinel = object()
-
-
-@dataclass
-class _OpenAIJsonSchema(WalkJsonSchema):
-    """Recursively handle the schema to make it compatible with OpenAI strict mode.
-
-    See https://platform.openai.com/docs/guides/function-calling?api-mode=responses#strict-mode for more details,
-    but this basically just requires:
-    * `additionalProperties` must be set to false for each object in the parameters
-    * all fields in properties must be marked as required
-    """
-
-    def __init__(self, schema: JsonSchema, strict: bool | None):
-        super().__init__(schema)
-        self.strict = strict
-        self.is_strict_compatible = True
-        self.root_ref = schema.get('$ref')
-
-    def walk(self) -> JsonSchema:
-        # Note: OpenAI does not support anyOf at the root in strict mode
-        # However, we don't need to check for it here because we ensure in pydantic_ai._utils.check_object_json_schema
-        # that the root schema either has type 'object' or is recursive.
-        result = super().walk()
-
-        # For recursive models, we need to tweak the schema to make it compatible with strict mode.
-        # Because the following should never change the semantics of the schema we apply it unconditionally.
-        if self.root_ref is not None:
-            result.pop('$ref', None)  # We replace references to the self.root_ref with just '#' in the transform method
-            root_key = re.sub(r'^#/\$defs/', '', self.root_ref)
-            result.update(self.defs.get(root_key) or {})
-
-        return result
-
-    def transform(self, schema: JsonSchema) -> JsonSchema:  # noqa C901
-        # Remove unnecessary keys
-        schema.pop('title', None)
-        schema.pop('default', None)
-        schema.pop('$schema', None)
-        schema.pop('discriminator', None)
-
-        if schema_ref := schema.get('$ref'):
-            if schema_ref == self.root_ref:
-                schema['$ref'] = '#'
-            if len(schema) > 1:
-                # OpenAI Strict mode doesn't support siblings to "$ref", but _does_ allow siblings to "anyOf".
-                # So if there is a "description" field or any other extra info, we move the "$ref" into an "anyOf":
-                schema['anyOf'] = [{'$ref': schema.pop('$ref')}]
-
-        # Track strict-incompatible keys
-        incompatible_values: dict[str, Any] = {}
-        for key in _STRICT_INCOMPATIBLE_KEYS:
-            value = schema.get(key, _sentinel)
-            if value is not _sentinel:
-                incompatible_values[key] = value
-        description = schema.get('description')
-        if incompatible_values:
-            if self.strict is True:
-                notes: list[str] = []
-                for key, value in incompatible_values.items():
-                    schema.pop(key)
-                    notes.append(f'{key}={value}')
-                notes_string = ', '.join(notes)
-                schema['description'] = notes_string if not description else f'{description} ({notes_string})'
-            elif self.strict is None:  # pragma: no branch
-                self.is_strict_compatible = False
-
-        schema_type = schema.get('type')
-        if 'oneOf' in schema:
-            # OpenAI does not support oneOf in strict mode
-            if self.strict is True:
-                schema['anyOf'] = schema.pop('oneOf')
-            else:
-                self.is_strict_compatible = False
-
-        if schema_type == 'object':
-            if self.strict is True:
-                # additional properties are disallowed
-                schema['additionalProperties'] = False
-
-                # all properties are required
-                if 'properties' not in schema:
-                    schema['properties'] = dict[str, Any]()
-                schema['required'] = list(schema['properties'].keys())
-
-            elif self.strict is None:
-                if (
-                    schema.get('additionalProperties') is not False
-                    or 'properties' not in schema
-                    or 'required' not in schema
-                ):
-                    self.is_strict_compatible = False
-                else:
-                    required = schema['required']
-                    for k in schema['properties'].keys():
-                        if k not in required:
-                            self.is_strict_compatible = False
-        return schema
-
-
-def _customize_request_parameters(model_request_parameters: ModelRequestParameters) -> ModelRequestParameters:
-    """Customize the request parameters for OpenAI models."""
-
-    def _customize_tool_def(t: ToolDefinition):
-        schema_transformer = _OpenAIJsonSchema(t.parameters_json_schema, strict=t.strict)
-        parameters_json_schema = schema_transformer.walk()
-        if t.strict is None:
-            t = replace(t, strict=schema_transformer.is_strict_compatible)
-        return replace(t, parameters_json_schema=parameters_json_schema)
-
-    return ModelRequestParameters(
-        function_tools=[_customize_tool_def(tool) for tool in model_request_parameters.function_tools],
-        allow_text_output=model_request_parameters.allow_text_output,
-        output_tools=[_customize_tool_def(tool) for tool in model_request_parameters.output_tools],
-    )
