@@ -8,6 +8,8 @@ from __future__ import annotations as _annotations
 from abc import ABC, abstractmethod
 from typing import Any, Generic, TypeVar
 
+from pydantic_ai.profiles import ModelProfile
+
 InterfaceClient = TypeVar('InterfaceClient')
 
 
@@ -41,6 +43,10 @@ class Provider(ABC, Generic[InterfaceClient]):
         """The client for the provider."""
         raise NotImplementedError()
 
+    def model_profile(self, model_name: str) -> ModelProfile | None:
+        """The model profile for the named model, if available."""
+        return None  # pragma: no cover
+
 
 def infer_provider(provider: str) -> Provider[Any]:
     """Infer the provider from the provider name."""
@@ -89,5 +95,17 @@ def infer_provider(provider: str) -> Provider[Any]:
         from .cohere import CohereProvider
 
         return CohereProvider()
+    elif provider == 'grok':
+        from .grok import GrokProvider
+
+        return GrokProvider()
+    elif provider == 'fireworks':
+        from .fireworks import FireworksProvider
+
+        return FireworksProvider()
+    elif provider == 'together':
+        from .together import TogetherProvider
+
+        return TogetherProvider()
     else:  # pragma: no cover
         raise ValueError(f'Unknown provider: {provider}')
