@@ -10,6 +10,8 @@ import httpx
 
 from pydantic_ai.exceptions import UserError
 from pydantic_ai.models import cached_async_http_client
+from pydantic_ai.profiles import ModelProfile
+from pydantic_ai.profiles.google import google_model_profile
 from pydantic_ai.providers import Provider
 
 try:
@@ -47,6 +49,9 @@ class GoogleVertexProvider(Provider[httpx.AsyncClient]):
     def client(self) -> httpx.AsyncClient:
         return self._client
 
+    def model_profile(self, model_name: str) -> ModelProfile | None:
+        return google_model_profile(model_name)
+
     @overload
     def __init__(
         self,
